#!/usr/bin/python2.4
# Generated by the protocol buffer compiler.  DO NOT EDIT!

from google.protobuf import descriptor
from google.protobuf import message
from google.protobuf import reflection
from google.protobuf import service
from google.protobuf import service_reflection
from google.protobuf import descriptor_pb2



_CONTACTGROUP = descriptor.Descriptor(
  name='ContactGroup',
  full_name='ContactGroup',
  filename='generated.proto',
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='id', full_name='ContactGroup.id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='group_name', full_name='ContactGroup.group_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='member', full_name='ContactGroup.member', index=2,
      number=3, type=11, cpp_type=10, label=3,
      default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],  # TODO(robinson): Implement.
  enum_types=[
  ],
  options=None)


_ADDRESSBOOK = descriptor.Descriptor(
  name='AddressBook',
  full_name='AddressBook',
  filename='generated.proto',
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='group', full_name='AddressBook.group', index=0,
      number=1, type=9, cpp_type=9, label=3,
      default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='phone', full_name='AddressBook.phone', index=1,
      number=2, type=9, cpp_type=9, label=1,
      default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='birthday', full_name='AddressBook.birthday', index=2,
      number=3, type=11, cpp_type=10, label=1,
      default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='favorite_flag', full_name='AddressBook.favorite_flag', index=3,
      number=4, type=8, cpp_type=7, label=1,
      default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='name', full_name='AddressBook.name', index=4,
      number=5, type=9, cpp_type=9, label=1,
      default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='email', full_name='AddressBook.email', index=5,
      number=6, type=9, cpp_type=9, label=1,
      default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],  # TODO(robinson): Implement.
  enum_types=[
  ],
  options=None)


_RESPONSE = descriptor.Descriptor(
  name='Response',
  full_name='Response',
  filename='generated.proto',
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='success', full_name='Response.success', index=0,
      number=1, type=8, cpp_type=7, label=1,
      default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='error_code', full_name='Response.error_code', index=1,
      number=2, type=5, cpp_type=1, label=1,
      default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='error_reason', full_name='Response.error_reason', index=2,
      number=3, type=9, cpp_type=9, label=1,
      default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],  # TODO(robinson): Implement.
  enum_types=[
  ],
  options=None)

import django_types_pb2

_CONTACTGROUP.fields_by_name['member'].message_type = _ADDRESSBOOK
_ADDRESSBOOK.fields_by_name['birthday'].message_type = django_types_pb2._DATE

class ContactGroup(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _CONTACTGROUP

class AddressBook(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _ADDRESSBOOK

class Response(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _RESPONSE


_ADDRESSSERVICE = descriptor.ServiceDescriptor(
  name='AddressService',
  full_name='AddressService',
  index=0,
  options=None,
  methods=[
  descriptor.MethodDescriptor(
    name='add_contact',
    full_name='AddressService.add_contact',
    index=0,
    containing_service=None,
    input_type=_ADDRESSBOOK,
    output_type=_RESPONSE,
    options=None,
  ),
  descriptor.MethodDescriptor(
    name='get_group',
    full_name='AddressService.get_group',
    index=1,
    containing_service=None,
    input_type=_CONTACTGROUP,
    output_type=_CONTACTGROUP,
    options=None,
  ),
])

class AddressService(service.Service):
  __metaclass__ = service_reflection.GeneratedServiceType
  DESCRIPTOR = _ADDRESSSERVICE
class AddressService_Stub(AddressService):
  __metaclass__ = service_reflection.GeneratedServiceStubType
  DESCRIPTOR = _ADDRESSSERVICE




#!python

#    Copyright (C) 2009  Zachary Walker
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Standard library imports
from time import time, sleep

# Third party imports
from protobuf.channel import SocketRpcChannel

# Module imports

# Constants
RPC_HOST = "localhost"
RPC_PORT = 8091

class ProtoBufRpcRequest(object):
    """ Class abstracting the Protocol Buffer RPC calls for a supplied
        service stub.
    """
  
    def __init__(self, service_stub_class, port=RPC_PORT, host=RPC_HOST):
        """ Contruct a new ProtoBufRpcRequest and return it.

            Accepted Arguments:
            service_stub_class -- (Service_Stub) The client side RPC
                                  stub class produced by protoc from
                                  the .proto file
            port -- (Integer) The port on which the service is running
                    on the RPC server. Default is value of
                    pond_client.RPC_PORT
            host -- (String) The hostname or IP address of the server
                    running the RPC service. Default is value of
                    pond_client.RPC_HOST
        """
        self.service_stub_class = service_stub_class
        self.port               = port
        self.host               = host
        
        #go through service_stub methods and add a wrapper function to 
        #this object that will call the method with only the msg as the arg
        for method in service_stub_class.GetDescriptor().methods:
            #Add service methods to the this object
            self.__dict__[method.name] = \
                lambda msg, timeout, service=self, method=method.name : service.call(
                    service_stub_class.__dict__[method], msg, timeout)
            
            #This is just a convenience that adds service methods to the
            #instance objects namespace.  No longer do it this way since
            #the  closures above are cleaner
            #self.__dict__[method.name] = \
            #    service_stub_class.__dict__[method.name]

    
    def call(self, rpc, msg, timeout):
        """ Save the object that has been created and return the response.
            Will timeout after timeout ms if response has not been received
            
            Accepted Arguments:
            timeout -- (Integer) ms to wait for a response before returning
        """
                
        # Define local callback function to handle RPC response
        # and initialize result dict
        result = {"done":False, "success":False, "response":None,
                        "error_msg":"timeout", "error_code":None}
        #Can't use just a simple boolean for 'done' because of the way it is
        #referenced in the callback.  The scope isn't clear and python
        #will redefine 'done' rather than change the value unless explicitly
        #defined as 'global done'
        #done = False
        def callback(request, response):
            result["response"]  = response
            result["done"]      = True
            result["error_msg"] = ""
            result["success"]   = True
        
        #Setup the RPC channel
        channel     = SocketRpcChannel(host=self.host, port=self.port)
        controller  = channel.newController()
        service     = self.service_stub_class(channel)
        
        #Make the RPC call and pass in a generic object with a
        #function called "run" linked to callback
        try:
            #Attempting to do something similar to a JAVA anonymous class
            #Create an instalce of a new generic type initialized with a
            #dict that has a run method pointing to the callback function
            rpc(service, controller, msg, type("", (), {"run":callback})())
        except Exception, info:
            print controller.error
        except:
            print controller.error
        end = time() + (timeout / 1000)
        
        #Tight loop waiting for timeout or callback to set done to True 
        while time() < end and not result["done"]:
            pass
        return result



#!/usr/bin/python2.4
# Generated by the protocol buffer compiler.  DO NOT EDIT!

from google.protobuf import descriptor
from google.protobuf import message
from google.protobuf import reflection
from google.protobuf import service
from google.protobuf import service_reflection
from google.protobuf import descriptor_pb2



_DATE = descriptor.Descriptor(
  name='Date',
  full_name='Date',
  filename='django_types.proto',
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='day', full_name='Date.day', index=0,
      number=1, type=5, cpp_type=1, label=2,
      default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='month', full_name='Date.month', index=1,
      number=2, type=5, cpp_type=1, label=2,
      default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='year', full_name='Date.year', index=2,
      number=3, type=5, cpp_type=1, label=2,
      default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='time', full_name='Date.time', index=3,
      number=4, type=11, cpp_type=10, label=1,
      default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],  # TODO(robinson): Implement.
  enum_types=[
  ],
  options=None)


_TIME = descriptor.Descriptor(
  name='Time',
  full_name='Time',
  filename='django_types.proto',
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='hour', full_name='Time.hour', index=0,
      number=1, type=5, cpp_type=1, label=2,
      default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='minute', full_name='Time.minute', index=1,
      number=2, type=5, cpp_type=1, label=2,
      default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='second', full_name='Time.second', index=2,
      number=3, type=5, cpp_type=1, label=2,
      default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],  # TODO(robinson): Implement.
  enum_types=[
  ],
  options=None)

import django_types_pb2

_DATE.fields_by_name['time'].message_type = _TIME

class Date(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _DATE

class Time(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _TIME




#!/usr/bin/python

# Set the enviroment variable Django looks at to find db settings
import os
from django.conf import ENVIRONMENT_VARIABLE
os.environ[ENVIRONMENT_VARIABLE] = 'settings'
#from django.core.management import setup_environ
#try:
#    import settings # Assumed to be in the same directory.
#    
#except ImportError:
#    import sys
#    sys.stderr.write("Error: Can't find the file 'settings.py' in the directory containing %r. It appears you've customized things.\nYou'll have to run django-admin.py, passing it your settings module.\n(If the file settings.py does indeed exist, it's causing an ImportError somehow.)\n" % __file__)
#    sys.exit(1)


from pbandj import pbandj

if __name__ == "__main__":
    #setup_environ(settings)
    from addressbook import models
    proto = models.genProto()
    pbandj.genMod(pbandj.DJANGO_PROTO)
    mod = __import__(proto.python_mod_name())
    port = 8091
    for service in proto.services:
        proto.services[service].start(mod, port)
        port += 1


#!/usr/bin/python
# Copyright (C) 2009  Las Cumbres Observatory <lcogt.net>
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''django_test.test_app.models.py - Django model used for unittesting
pbandj

Authors: Zach Walker (zwalker@lcogt.net)
Dec 2009
'''

from django.db import models


class OneOfEverything(models.Model):
    bool_test = models.BooleanField()
    char_test = models.CharField(max_length=10)
    comma_test = models.CommaSeparatedIntegerField(max_length=10)
    date_test = models.DateField()
    date_time_test = models.DateTimeField()
    decimal_test = models.DecimalField(decimal_places=2, max_digits=4)
    email_test = models.EmailField()
    file_test = models.FileField(upload_to='.')
    file_path_test = models.FilePathField()
    float_test = models.FloatField()
    image_test = models.ImageField(upload_to='.')
    int_test = models.IntegerField()
    ip_test = models.IPAddressField()
    null_bool_test = models.NullBooleanField()
    pos_int_test = models.PositiveIntegerField()
    pos_sm_int_test = models.PositiveSmallIntegerField()
    slug_test = models.SlugField()
    sm_int_test = models.SmallIntegerField()
    text_test = models.TextField()
    time_test = models.TimeField()
    url_test = models.URLField()
    xml_test = models.XMLField()


class Simple(models.Model):
    val = models.IntegerField()


class EnumTest(models.Model):
    test_enum_choices = (('Val1', 'Long Name Val 1'),
                         ('Val2', 'Long Name Val 2'))
    enum_test = models.CharField(max_length = 20, choices=test_enum_choices)


class ForeignKeyTest(models.Model):
    fkey_test = models.ForeignKey(Simple)


class ManyToManyTest(models.Model):
    test_val = models.IntegerField()
    m2m_test = models.ManyToManyField(Simple)

class ManyToManyThroughTest(models.Model):
    test_val = models.IntegerField()
    m2m_test = models.ManyToManyField(Simple, through='M2MAssocTest')
    
class M2MAssocTest(models.Model):
    assoc_test = models.IntegerField()
    simple_fk = models.ForeignKey(Simple)
    m2m_fk = models.ForeignKey(ManyToManyThroughTest)



#!/usr/bin/python
# Copyright (C) 2009  Las Cumbres Observatory <lcogt.net>
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''type_map_base.py - Type mapping definitions for Django fields to
protocol buffer fields

Authors: Zach Walker (zwalker@lcogt.net)
Dec 2009
'''

from django.db import  models


def _make_type(name, ptype):
    return type(name, (), {'name': name,
                           'ptype': ptype,
                           '__eq__': lambda x,y: isinstance(y, x.__class__) and x.ptype == y.ptype,
                           '__ne__': lambda x,y: not x.__eq__(y)})

PB_TYPE_STRING  = _make_type('PB_TYPE_STRING', 'string')
PB_TYPE_DOUBLE  = _make_type('PB_TYPE_DOUBLE','double')
PB_TYPE_INT32   = _make_type('PB_TYPE_INT32','int32')
PB_TYPE_INT64   = _make_type('PB_TYPE_INT64','int64')
PB_TYPE_UINT32  = _make_type('PB_TYPE_UINT32','uint32')
PB_TYPE_UINT64  = _make_type('PB_TYPE_UINT64','uint64')
PB_TYPE_SINT32  = _make_type('PB_TYPE_SINT32','sint32')
PB_TYPE_SINT64  = _make_type('PB_TYPE_SINT64','sint64')
PB_TYPE_FINT32  = _make_type('PB_TYPE_FINT32','fixed32')
PB_TYPE_FINT64  = _make_type('PB_TYPE_FINT64','fixed64')
PB_TYPE_SFINT32 = _make_type('PB_TYPE_SFINT32','sfixed32')
PB_TYPE_SFINT64 = _make_type('PB_TYPE_SFINT64','sfixed64')
PB_TYPE_FLOAT   = _make_type('PB_TYPE_FLOAT','float')
PB_TYPE_BOOL    = _make_type('PB_TYPE_BOOL','bool')
PB_TYPE_BYTES   = _make_type('PB_TYPE_BYTES','bytes')


#PB_TYPE_STRING  = type('PB_TYPE_STRING', (), {'ptype': 'string'})
#PB_TYPE_DOUBLE  = type('PB_TYPE_DOUBLE', (), {'ptype': 'double'})
#PB_TYPE_INT32   = type('PB_TYPE_INT32', (), {'ptype': 'int32'})
#PB_TYPE_INT64   = type('PB_TYPE_INT64', (), {'ptype': 'int64'})
#PB_TYPE_UINT32  = type('PB_TYPE_UINT32', (), {'ptype': 'uint32'})
#PB_TYPE_UINT64  = type('PB_TYPE_UINT64', (), {'ptype': 'uint64'})
#PB_TYPE_SINT32  = type('PB_TYPE_SINT32', (), {'ptype': 'sint32'})
#PB_TYPE_SINT64  = type('PB_TYPE_SINT64', (), {'ptype': 'sint64'})
#PB_TYPE_FINT32  = type('PB_TYPE_FINT32', (), {'ptype': 'fixed32'})
#PB_TYPE_FINT64  = type('PB_TYPE_FINT64', (), {'ptype': 'fixed64'})
#PB_TYPE_SFINT32 = type('PB_TYPE_SFINT32', (), {'ptype': 'sfixed32'})
#PB_TYPE_SFINT64 = type('PB_TYPE_SFINT64', (), {'ptype': 'sfixed64'})
#PB_TYPE_FLOAT   = type('PB_TYPE_FLOAT', (), {'ptype': 'float'})
#PB_TYPE_BOOL    = type('PB_TYPE_BOOL', (), {'ptype': 'bool'})
#PB_TYPE_BYTES   = type('PB_TYPE_BYTES', (), {'ptype': 'bytes'})


DJ2PB = {models.CharField: PB_TYPE_STRING,
         models.DecimalField: PB_TYPE_DOUBLE,
         models.IntegerField: PB_TYPE_INT32,
         models.FloatField: PB_TYPE_FLOAT,
         models.DateTimeField: PB_TYPE_STRING,
         models.DateField: PB_TYPE_STRING,
         models.BooleanField: PB_TYPE_BOOL,
         models.NullBooleanField: PB_TYPE_BOOL,
         models.CommaSeparatedIntegerField: PB_TYPE_STRING,
         models.EmailField : PB_TYPE_STRING,
         models.FileField: PB_TYPE_STRING,
         models.FilePathField:  PB_TYPE_STRING,
         models.ImageField: PB_TYPE_STRING,
         models.IPAddressField: PB_TYPE_STRING,
         models.PositiveIntegerField: PB_TYPE_UINT32,
         models.PositiveSmallIntegerField: PB_TYPE_UINT32,
         models.SlugField: PB_TYPE_STRING,
         models.SmallIntegerField: PB_TYPE_INT32,
         models.TextField: PB_TYPE_STRING,
         models.TimeField: PB_TYPE_STRING,
         models.URLField: PB_TYPE_STRING,
         models.XMLField: PB_TYPE_STRING,
         models.AutoField: PB_TYPE_INT32,
         models.ForeignKey: PB_TYPE_INT32,
         # TODO: Add a ManyToManyField mapping
         models.ManyToManyField: PB_TYPE_INT32 
         }

PB2DJ = dict([(DJ2PB[key], key) for key in DJ2PB])



from django.db import models
from pbandj.model import ProtocolBuffer as Proto
from pbandj import type_map
from pbandj.conversion import Converter

""" Alias for ProtocolBuffer.Field """
Field = Proto.Field


class AddressBook(models.Model):
    """ Address Book contact class extending Django model
    """
    name            = models.CharField(max_length=25)
    phone           = models.CharField(max_length=25)
    email           = models.CharField(max_length=25)
    birthday        = models.DateField()
    favorite_flag   = models.BooleanField()


class ContactGroup(models.Model):
    """ Address Book contact class extending Django model
    """
    group_name            = models.CharField(max_length=25)
    member_name           = models.ForeignKey(AddressBook)
 
# The protocol buffer object    
pbuf = Proto()

def onAddContact(service_impl, controller, request, done):
    """ Handler method for the add_contact RPC described below """
    print "In add contact"
    from generated_pb2 import Response
    resp = Response()
    print "Converting to django"
    converter = Converter()
    try:
        contact = converter.toDjangoObj(pbuf, request)
    except Exception, e:
        print e
    contact.save()
    for group in request.group:
        cont_grp = ContactGroup()
        cont_grp.group_name = group
        cont_grp.member_name = contact
        
        cont_grp.save()
    resp.success = True
    print "About to exit"
    done.run(resp)

def onGetGroup(service_impl, controller, request, done):
    """ Handler method for the get_group RPC method described below """
    print "In onGetGroup"
    from generated_pb2 import ContactGroup as PbGroup
    members = ContactGroup.objects.filter(group_name=request.group_name)
    resp = PbGroup()
    resp.group_name = request.group_name
    print "Adding memebers to response msg"
    for contact in members:
        print "Converting"
        converter = Converter()
        converter.toProtoMsg(pbuf, contact.member_name, resp.member.add())
        print "Done converting"
    done.run(resp)
    
def genProto():
    """ Generate and return the protocol buffer description object """
    
    # Create a generic response message
    rsp_msg = Proto.Message("Response")
    rsp_msg.addField(Field(Field.OPTIONAL, "success",
                           type_map.base.PB_TYPE_BOOL))
    rsp_msg.addField(Field(Field.OPTIONAL, "error_code",
                           type_map.base.PB_TYPE_INT32))
    rsp_msg.addField(Field(Field.OPTIONAL, 'error_reason',
                           type_map.base.PB_TYPE_STRING))
    # Add the respons message to the proto
    pbuf.addMessage(rsp_msg)
    
    # Create a contact message from the AddressBook model
    cont_msg = type_map.genMsg('AddressBook',AddressBook,exclude=["id"])
    cont_msg.addField(Field(Field.REPEATED,'group',
                            type_map.base.PB_TYPE_STRING))
    pbuf.addMessage(cont_msg)
    
    # Create a service accepting contact messages
    pbuf.addRpc("AddressService",'add_contact',
                     cont_msg, rsp_msg, onAddContact)
    
    # Create a group message from the ContactGroup Model
    grp_msg = type_map.genMsg('ContactGroup',ContactGroup,exclude=["member_name"])
    grp_msg.addField(Field(Field.REPEATED,'member', cont_msg))
    pbuf.addMessage(grp_msg)
    
    # Create a service returning memebers of a contact group
    pbuf.addRpc("AddressService", 'get_group',
                    grp_msg, grp_msg, onGetGroup)
    return pbuf


#!/usr/bin/python
# Copyright (C) 2009  Las Cumbres Observatory <lcogt.net>
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''model.py - A class structure mapping the meta data needed to construct
a protocol buffer .proto file
                         
Authors: Zachary Walker (zwalker@lcogt.net)
July 2009
'''

# Standard library imports
import threading
from datetime import date, datetime
from socket import gethostname

# Third party imports
from protobuf.server import SocketRpcServer

# Module imports


class ProtocolBuffer(object):
    """ A class containing data describing a Protocol
        Buffer .proto file and it's mapping to Django
        objects
    """ 
    
    
    
    class Enumeration(object):
        
        def __init__(self, name, values=None, doc=None):
            '''Create an Enumeration object with given name and list of values
            to be enumerated
            '''
            self.name = name.lower().capitalize()
            # Holds Protobuf enum val name to int mapping
            name2val = {}
            # Holds Django val to int mapping
            dj2val = {}
            #self.values = {}
            if values != None:
                # Map django choices to int values
                dj2val = [(str(values[i]), i) for i in range(len(values))]
                dj2val += [(val, key) for key,val in dj2val]
                # Create a enum to val mapping
                name2val = [[name.upper() + '_' + str(values[i]), i] for i in range(len(values))]
                # Format the enum constant names
                for pair in name2val:
                    name = pair[0]
                    if name[0].isdigit():
                        name = '_' + name
                    name = name.replace('.', '_')
                    name = name.replace('-', '_')
                    name = name.upper()
                    pair[0] = name
                self.name2val = dict(name2val)
                self.dj2val = dict(dj2val)
                self.doc = doc
        
        def __eq__(self, other):
            if not isinstance(other, self.__class__):
                return False
#            print "Enum compare", (self.name == other.name ,
#                     self.name2val == other.name2val , 
#                     self.dj2val == other.dj2val)
            return (self.name == other.name and
                     self.name2val == other.name2val and 
                     self.dj2val == other.dj2val)
            
        def __ne__(self, other):
            return not self.__eq__(other)
        
        # Hacked hash.  Enums aren't imutable as implemented.
        # Be careful using hash
        def __hash__(self):
            return hash(self.name) ^ hash(tuple(self.name2val.items())) ^ hash(tuple(self.dj2val.items()))
        
        def __str__(self):
            if self.doc is None:
                proto = ''
            else:
                proto = "// %s\n" % str(self.doc).strip()
            proto += 'enum %s {\n' % self.name
            for key in self.name2val:
                if(type(key) == str):
                    proto += '\t%s = %d;\n' % (key, self.name2val[key])
            proto += '}\n\n'
            return proto
             
            
    
    class Field(object):
        """ A class containing data describing a
            protocol buffer message field and any
            mapping to a related Django field
        """
        
        OPTIONAL = "optional"
        REQUIRED = "required"
        REPEATED = "repeated"
        
        def __init__(self, usage, name, pb_type, dj_type=None):
            """
                Accepted Arguments:
                usage -- (String) Used of this field in the .proto file
                         OPTIONAL = "optional"
                         REQUIRED = "required"
                         REPEATED = "repeated"
                name-- (String) Name of this field
                pb_type -- (String) The Protocol Buffer type for this
                           message field
                dj_type -- (String) The Django type that this field is
                           mapped to.  None if this field isn't mapped
                           to Django
            """
            self.name    = name
            self.usage   = usage
            self.dj_type = dj_type
            self.pb_type = pb_type
            
        def __eq__(self, other):
            if not isinstance(other, self.__class__):
                return False
            return (self.name == other.name ,
                    self.usage == other.usage and
                    self.dj_type == other.dj_type and
                    self.pb_type == other.pb_type)
        
        def __ne_(self, other):
            return not self.__eq__(other)
        
        # Hacked hash.  Fields aren't immutable as implemented.
        # Be careful using hash
        def __hash__(self):
            return hash(self.name) ^ hash(self.usage) ^ hash(self.dj_type) ^ hash(self.pb_type)
             
       
             
    class Message(object):
        """ A class containing data describing a protocol buffer
            message and mapping to a Django model type if
            supplied. If a Django model type has been associated
            with a Message then all fields of the message with
            Django types can be translated between protocol
            buffer and django objects
        """
        
        def __init__(self, msg_name, django_model_class=None):
            """
                Accepted Arguments:
                msg_name -- (String) Name of this protocol
                            buffer message
                django_model_class -- (Django.db.models.model)
                                      Django class that this message
                                      will be mapped to.
                                      If None it will not be mapped 
            """
            self.name               = msg_name
            self.fields             = []
            self.django_model_class = django_model_class
            self.mapped_fields      = []
            self.message_fields     = []
            self.enums              = []
            self.__iadd__           = None
        
        def __eq__(self, other):
            if not isinstance(other, self.__class__):
                return False
            if self.name != other.name: return False
            if len(self.fields) != len(other.fields): return False
            if len(self.mapped_fields) != len(other.mapped_fields): return False
            if len(self.message_fields) != len(other.message_fields): return False
            if len(self.enums) != len(other.enums): return False
            for field in self.fields:
                if other.fields.count(field) != self.fields.count(field):
                    return False #break
            if self.django_model_class != other.django_model_class: return False
            for field in self.mapped_fields:
                if other.mapped_fields.count(field) != self.mapped_fields.count(field):
                    return False
            for field in self.message_fields:
                if other.message_fields.count(field) != self.message_fields.count(field):
                    return False
            for enum in self.enums:
                if other.enums.count(enum) != self.enums.count(enum):
                    return False
            
            return True
            
        def __ne__(self, other):
            return not self.__eq__(other)
        
        # Hacked hash.  Messages aren't immutable as implemented.
        # Be careful using hash
        def __hash__(self):
            return hash(self.name) ^ hash(tuple(self.mapped_fields))
        
        def __add__(self, other):
            if not isinstance(other, self.__class__):
                raise Exception("Can't add type %s to a %s" % 
                                (type(other), type(self)))
            s1 = set(self.message_fields + self.fields + self.enums)
            s2 = set(other.message_fields + other.fields + other.enums)
            diff = s1.symmetric_difference(s2)
            self_names = set([obj.name for obj in s1.intersection(diff)])
            other_names = set([obj.name for obj in s2.intersection(diff)])
            colissions = self_names.intersection(other_names)
            if colissions != set():
                raise Exception("Namespace collision between messages on %s" % colissions)
            new_msg = ProtocolBuffer.Message(self.name, self.django_model_class)  
            new_msg.mapped_fields = self.mapped_fields
            new_msg.fields = list(set(self.fields).union(set(other.fields)))
            new_msg.message_fields = list(set(self.message_fields).union(set(other.message_fields)))
            new_msg.enums = list(set(self.enums).union(set(other.enums)))
            return new_msg
            
            
        def addField(self, *fields):
            """ Add a field to this Protocol Buffer message   
                Accepted Arguments:
                field -- (ProtocolBuffer.Field) A field to be added to
                         the message
            """
            # Check if this field is mapped to a django type
            for field in fields:
                if field.dj_type is None:
                    # Has the field already been mapped to a message
                    if type(field.pb_type) == ProtocolBuffer.Message:
                        self.message_fields.append(field)
                    else:
                        self.fields.append(field)
                else:
                    self.mapped_fields.append(field)

        @staticmethod
        def _write_proto_field(field, field_num):
            field_str = ''
            if type(field.pb_type) == ProtocolBuffer.Message:
                field_type = field.pb_type.name
            elif type(field.pb_type) == ProtocolBuffer.Enumeration:
                field_type = field.pb_type.name
            else:
                field_type = field.pb_type.ptype
            field_str = field_str + "\t%s %s %s = %d;\n"
            field_str %= (field.usage,
                        field_type,
                        field.name, field_num)
            return field_str
        
        def __str__(self):
            """ Produces a .proto file message declaration
            """
            #fields = self.fields + self.mapped_fields + self.message_fields
            #field_base = 1
            field_num = 1
            if self.django_model_class != None:
                message = "// Generated from Django model %s\n" % self.django_model_class.__name__
            else:
                message = ""
            message +=   "message " + self.name + " {\n"
            for enum in self.enums:
                enum_str = str(enum)
                for line in enum_str.splitlines():
                    message += "\t%s\n" % line
                #message += str(enum)
#            for i in range(len(fields)):
#                field = fields[i]
#                if type(field.pb_type) == ProtocolBuffer.Message:
#                    field_type = field.pb_type.name
#                elif type(field.pb_type) == ProtocolBuffer.Enumeration:
#                    field_type = field.pb_type.name
#                else:
#                    field_type = field.pb_type.ptype
#                message = message + "    %s %s %s = %d;\n"
#                message %= (field.usage,
#                            field_type,
#                            field.name, i + field_base)
            if len(self.mapped_fields) > 0:# and not self.django_model_class is None: 
                message += "\t// Fields mapped to django model %s\n" % self.django_model_class.__name__ 
                for field in self.mapped_fields:
                    message += ProtocolBuffer.Message._write_proto_field(field, field_num)
                    field_num += 1
            if (len(self.message_fields) + len(self.fields)) > 0:
                message += "\t// Unmapped fields\n"
                for field in self.message_fields:
                    message += ProtocolBuffer.Message._write_proto_field(field, field_num)
                    field_num += 1
                for field in self.fields:
                    message += ProtocolBuffer.Message._write_proto_field(field, field_num)
                    field_num += 1
            message = message + "}\n"
            return message
        
        
        
    class Service(object):
        """ A class containing data describing a protocol buffer
            service
        """
        
        class ServiceThread(threading.Thread):
            
            def __init__(self, module, service, port):
                threading.Thread.__init__(self)
                self.module = module
                self.service = service
                self.port = port
                
#            def _createHandler(self, func):
#                def handler(service_impl, controller, request, done):
#                    try:
#                        result = func(request)
#                    except Exception, e:
#                        controller.handleError(e, e.__str__())
#                    done.run(result)
                
            def run(self):
                """ Start service thread """
                # Create a server instance
                server = SocketRpcServer(self.port,host='')
                service_class = self.module.__dict__[self.service.name]
                handlers = {}
                
                # Link each method to its handler
                for method in self.service.methods:
                    #handlers[method.name] = self._createHandler(method.handler)
                    handlers[method.name] = method.handler
                
                # Extend Service class and add handlers    
                newservice = type(self.service.name + "Impl",
                                  (service_class,), handlers)
                server.registerService(newservice())
                
                # Start the server
                server.run();
        
        
        def __init__(self, name):
            """
                Accepted Arguments:
                name -- (String) The name of the protocol buffer
                        service
                port -- (Integer) The port number on which this
                        service will be run
            """
            self.name    = name
            self.methods = []
        
        
        def addRPCMethod(self, rpc_method_name, in_msg_type,
                     out_msg_type, rpc_handler, doc=None):
            """ Generate a Protocol Buffer RPC
                rpc_method_name -- (String) Name of the rpc method being
                                   created within the service
                in_msg_type -- (ProtocolBuffer.Message) 
                               A ProtocolBuffer.Message describing the in
                               msg arg to the rpc call
                out_msg_type -- (ProtocolBuffer.Message)
                                A ProtocolBuffer.Message describing the
                                response msg to the rpc call
                rpc_handler -- (Method) A reference to a callable method
                               with signature (service_impl, controller,
                               request, done) that will be handle the rpc
                               call
            """
            rpc = ProtocolBuffer.RPCMethod(rpc_method_name,
                                       in_msg_type,
                                       out_msg_type,
                                       rpc_handler,
                                       doc)
            self.methods.append(rpc)
            
        def start(self, module, port, daemon=False):
            """ Start service in a new thread """
            thread = ProtocolBuffer.Service.ServiceThread(module, self, port)
            thread.setDaemon(daemon)
            thread.start()
            
    
    
    
    class RPCMethod(object):
        """ A class encapsulation the description of a Protocol Buffer
            RPC declaration 
        """
        
        def __init__(self, name, in_msg_type, out_msg_type, callable, doc=None):
            """
                Accepted Arguments:
                name -- (String) The name of the RPC method
                in_msg_type -- (ProtocolBuffer.Message) The type of
                               message passed as an arg to this RPC
                               method
                out_msg_type -- (ProtocolBuffer.Message) The type of
                                message to be returned from the RPC
                                method
                callable -- (function) A function to be called to
                            handle the RPC call, signature 
                            (service_impl, controller, request, done)
            """
            self.name         = name
            self.in_msg_type  = in_msg_type
            self.out_msg_type = out_msg_type
            self.handler      = callable
            if doc is None or len(doc) == 0:
                self.doc = callable.__doc__
            else:
                self.doc          = doc
        
    
    
    def __init__(self, module_name='generated'):
        """
            AcceptedArguments:
            module_name -- (String) The name of the python module that
                           will be generated for this Protocol Buffer
        """
        self.module_name = module_name
        self.imports     = []
        self.messages    = {}
        self.services    = {}
        self.enum        = {}
        
    
    def python_mod_name(self):
        return self.module_name + "_pb2"
    
    def proto_filename(self):
        return self.module_name + ".proto"
          
    def addMessage(self, message, user_type=False):
        """ Add a message to this Protocol Buffer definition.  Also adds any
        messages that are used as fields of this message.  Message fields added
        after the message is added to the protocol buffer must be added to the
        protocol buffer directly using addMessage.  An Exception will be raised
        if a field of the root message is a message of the same name but different
        type of a message already included in the protocol buffer.  A name 
        collision on the root message will be ignored and the existing messasge
        will be overwritten.
            Accepted Arguments:
            message        -- (ProtocolBuffer.Message) The message to
                              be added to the Protcol Buffer
            user_type -- If true no mapping will be added for django to
                protobuf conversion.  Conversion can still be done by calling
                toProtoMsg with dest_obj set to the desired type.  This allows
                multiple messages mapped to the same Django type while only having
                a single default message used for auto conversion.
        """
        # Check if there is already a message with the same name
        orig_msg = self.messages.get(message.name, None)
        if self.messages.has_key(message.name):
            #print "Collision on %s" % message.name
            if self.messages[message.name] != message:
                if (self.messages[message.name].mapped_fields == message.mapped_fields):
                    #print "Merging messages"
                    message = self.messages[message.name] + message
                else:
                    raise Exception("There is already a message named %s whose mapped fields differs from the added message" %
                                    message.name)
        
        # Check for a mapping from the django type diffing from the added message        
        if (self.messages.has_key(message.django_model_class) and
            self.messages[message.django_model_class] != message and
            not user_type):
            if (self.messages[message.django_model_class] != message and
                self.messages[message.django_model_class].mapped_fields == message.mapped_fields):
                #print "Merging messages"
                message = self.messages[message.name] + message
            else:
                raise Exception("There is already a message mapped to django type %s whose mapped fields differ from the added message" %
                                message.django_model_class.__name__)
        if orig_msg != message:
            # Add mapping by message name
            self.messages[message.name] = message
            # Add mapping by django model class
            # TODO: Problem here if two messages are added with the same django_model_class
            #       With different fields within the message.  Each message added would override
            #       the previous one.
            if not message.django_model_class is None and user_type == False:
                    self.messages[message.django_model_class] = message
    
            # Add message mapping for fields of type message 
            for field in message.mapped_fields:
                if (isinstance(field.pb_type, ProtocolBuffer.Message) and
                    not field.pb_type.django_model_class is None):
                    #print "In %s.%s Adding type %s, %s" % (message.name, field.name, field.pb_type.name, user_type)
                    self.addMessage(field.pb_type, user_type=user_type)
            for field in message.message_fields:
                #print "In %s.%s Adding type %s, %s" % (message.name, field.name, field.pb_type.name, True)
                self.addMessage(field.pb_type, user_type=True)
            
    
    
    def addRpc(self, service_name, method_name, in_msg, out_msg, handler, doc=None):
        ''' Add a new method to the specified service or create the service
        if it doesn't already exist.  Service will take imput type in_msg
        and return type out_msg.  The optional doc argument will be inserted
        in the genreated .proto file along with the service declaration.
        '''
        service = self.services.get(service_name)
        if service:
            service.addRPCMethod(method_name, in_msg, out_msg, handler, doc)
        else:
            service = ProtocolBuffer.Service(service_name)
            service.addRPCMethod(method_name, in_msg, out_msg, handler, doc)
            self.services[service_name] = service
    
    
    def __str__(self):
        """ Produce a .proto file string for this Protocol Buffer
            instance
        """
        out= "//Protocol Buffer file generated by pbandj.\n\n"
        imported_msgs = []
        for proto in self.imports:
            imported_msgs += proto.messages.values()
            if type(proto) is ProtocolBuffer:
                out += 'import "%s";\n' % proto.proto_filename()
            else:
                out += 'import "%s";\n' % proto
        if len(self.imports) > 0:
            out += "\n"
        for key in self.services.keys():
            service = self.services[key]
            out += "service " + service.name + " {\n"
            for method in service.methods:
                if isinstance(method.doc, str) and len(method.doc) > 0:
                    for doc_line in method.doc.strip().splitlines():
                        out += "\t// %s\n" % doc_line.strip()
                out += "\trpc " + method.name + " (" + \
                       method.in_msg_type.name + ") returns (" + \
                       method.out_msg_type.name + ");\n\n"
            out += "}\n" 
        
        #print imported_msgs
        # Only include messages that are not defined in an import
        for key in self.messages.keys():
            if isinstance(key,str):
                if not self.messages[key] in imported_msgs:
                    out += "%s\n" % str(self.messages[key])
            
        return out
        
    
        



#
#from pbandj.model import ProtocolBuffer
#from django_test.test_app import models
#
#protobuf = ProtocolBuffer()
#
#def generateMessages():
#    one_of_everything = protobuf.genMsg('OneOfEverything',
#                                        models.OneOfEverything)
#    protobuf.addMessage(one_of_everything)
    


#!/usr/bin/python
# Copyright (C) 2009  Las Cumbres Observatory <lcogt.net>
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''__init__.py 

This package contains classes providing a mappings between Django models
and Protocol Buffer messages and RPC services

Authors: Zach Walker (zwalker@lcogt.net)
Dec 2009
'''

# Point django settings to dummy settings if not have already been set
import os
if not os.environ.has_key('DJANGO_SETTINGS_MODULE'):
    print "WARNING: Using dummy django settings"
    os.environ['DJANGO_SETTINGS_MODULE'] = 'pbandj.dummy_settings'


#!/usr/bin/python
# Copyright (C) 2009  Las Cumbres Observatory <lcogt.net>
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''type_map.py - Type mapping definitions for Django fields that
don't map onto protocol buffer field types directly.

Authors: Zach Walker (zwalker@lcogt.net)
Dec 2009
'''

from __future__ import division
from datetime import datetime, time


import type_map_base as base
from django.db import models
from model import ProtocolBuffer


def Time():
    try:
        Time.instance
    except:
        time = ProtocolBuffer.Message('Time')
        time.addField(ProtocolBuffer.Field(ProtocolBuffer.Field.REQUIRED,
                                           'hour',
                                           base.PB_TYPE_INT32))
        time.addField(ProtocolBuffer.Field(ProtocolBuffer.Field.REQUIRED,
                                           'minute',
                                           base.PB_TYPE_INT32))
        time.addField(ProtocolBuffer.Field(ProtocolBuffer.Field.REQUIRED,
                                           'second',
                                           base.PB_TYPE_INT32))
        #time.addField(ProtocolBuffer.Field(ProtocolBuffer.Field.REQUIRED,
        #                                  'micro',
        #                                   base.PB_TYPE_INT32))
        Time.instance = time
    return Time.instance



def Date():
    try:
        Date.instance
    except:
        date = ProtocolBuffer.Message('Date')
        date.addField(ProtocolBuffer.Field(ProtocolBuffer.Field.REQUIRED,
                                          'day',
                                          base.PB_TYPE_INT32))
        date.addField(ProtocolBuffer.Field(ProtocolBuffer.Field.REQUIRED,
                                          'month',
                                          base.PB_TYPE_INT32))
        date.addField(ProtocolBuffer.Field(ProtocolBuffer.Field.REQUIRED,
                                          'year',
                                          base.PB_TYPE_INT32))
        date.addField(ProtocolBuffer.Field(ProtocolBuffer.Field.OPTIONAL,
                                          'time',
                                          Time()))
        Date.instance = date
    return Date.instance

#def _buildFile():
#    try:
#        _buildFile.file
#    except:
#        file = ProtocolBuffer.Message('File')
#        file.addField(ProtocolBuffer.Field(ProtocolBuffer.Field.REQUIRED,
#                                           'name',
#                                           base.PB_TYPE_STRING))
#        file.addField(ProtocolBuffer.Field(ProtocolBuffer.Field.REQUIRED,
#                                           'path',
#                                           base.PB_TYPE_STRING))
#        file.addField(ProtocolBuffer.Field(ProtocolBuffer.Field.OPTIONAL,
#                                           'data',
#                                           base.PB_TYPE_BYTES))
#        file.addField(ProtocolBuffer.Field(ProtocolBuffer.Field.OPTIONAL,
#                                           'size',
#                                           base.PB_TYPE_INT32))
#        file.addField(ProtocolBuffer.Field(ProtocolBuffer.Field.OPTIONAL,
#                                           'url',
#                                           base.PB_TYPE_STRING))
#        _buildFile.file = file
#    return _buildFile.file

def _buildDJ2PB():
    map = base.DJ2PB
    map[models.TimeField] = Time()
    map[models.DateField] = Date()
    map[models.DateTimeField] = Date()
    #map[models.FileField] = _buildFile()
    return map

def _buildDjangoProto():
    pb = ProtocolBuffer('django_types')
    pb.addMessage(Date())
    pb.addMessage(Time())
    #pb.addMessage(_buildFile())
    return pb

DJ2PB = _buildDJ2PB() 
PB2DJ = [(val,key) for (key,val) in DJ2PB.items()]
DJANGO_PROTO = _buildDjangoProto()

# TODO: Add a recursion depth limit
# TODO: Prevent circular recursion        
def genMsg(msg_name, django_model_class, include=[], exclude=[], recurse_fk=True, no_recurse=[]):
    """ Create a protocol buffer message from a django type.
        By default all args will be included in the new message.
        If recurse_fk is set messages will also be created for foreign key
        types.  A list of generated messages will be returned.
        Accepted Args:
        msg_name -- (String) The name of the message
        django_model_class -- (Type) A reference to the django
                              model type
        included -- (List) A list of field names from the model to
                  be included in the mapping
        exclude -- (List) A list of field names to exclude from the
                  model.  Only used if included arg is []
        recurse_fk -- Flag for recursing into foreign key fields if true or 
                   using pk if false
        no-recurse -- List of field names which should not be recursed.
                      Only used if recurse_fk=True
    """
    Field = ProtocolBuffer.Field
    Enum = ProtocolBuffer.Enumeration
    #messages = {}
    root_msg = ProtocolBuffer.Message(msg_name, django_model_class)
    #messages[msg_name] = root_msg
    
    # Local fields
    django_class_fields = [field for field in 
                           django_model_class._meta.local_fields +
                           django_model_class._meta.many_to_many]
    django_field_by_name = {}
    
    # Iterate through Django fields and init dict that allows
    # lookup of fields by name.
    for field in django_class_fields:
        django_field_by_name[field.name] = field
    #[django_field_by_name.__setitem__(field.name, field) for field in
    #    django_class_fields]
    
    # Remove excluded fields or add included fields
    field_set = set()
    if include != []: 
        # Assert that the fields all exist
        include_set = set(include)
        assert include_set == set(django_field_by_name.keys()) & include_set
        field_set = include_set
    else:
        useable_field_names = set(django_field_by_name.keys()) - set(exclude)
        field_set = useable_field_names
    
    # Add a pb field to the pb msg for each django field in the set
    new_fields = []
    for field_name in field_set:
        field = django_field_by_name[field_name]
        #pb_usage = Field.OPTIONAL
        #pb_type = None
        #pb_field_name = field.name
        if isinstance(field, models.ForeignKey):
            if recurse_fk and not field.name in no_recurse:
                # Create a new msg type from the parent table in the fk relation
                fk_dj_model = field.rel.to
                if isinstance(fk_dj_model, str):
                    # TODO: Handle ForeignKey supplied as a str
                    #print fk_dj_model
                    assert False
                    #fk_dj_model = models.__dict__[fk_dj_model]
                pb_type = genMsg(fk_dj_model.__name__, fk_dj_model, recurse_fk=recurse_fk)
                new_fields.append(Field(Field.OPTIONAL, field.name,
                                        pb_type, field))
                # TODO: Change msg name to avoid name collision at the ProtocolBuffer level
            #else:
                # Use the pk from the foreign key relation as the value
            pb_type = DJ2PB.get(type(field))()
            new_fields.append(Field(Field.OPTIONAL, field.name + '_id',
                                    pb_type, field))
        elif isinstance(field, models.ManyToManyField):
            # If there is a 'through' model, add a repeated field for the through
            # data even if recurse_fk == False. If there is no through model and
            # recrse_fk == False then add repeated reference to fk object pk's.
            #pb_usage = Field.REPEATED
            if not field.rel.through is None:
                # There is data associated with the relation
                # Generate a msg from assoc model excluding fk reference
                # back to the parent type to prevent circular references
                m2m_dj_model = field.rel.through_model
                recur_rel_fields = [] # Fields which would cause a recursive relation
                for assoc_field in m2m_dj_model._meta._fields():
                    if (isinstance(assoc_field, models.ForeignKey) and
                        assoc_field.rel.to == django_model_class):
                        recur_rel_fields.append(assoc_field.name)
                pb_type = genMsg(m2m_dj_model.__name__, m2m_dj_model, recurse_fk=recurse_fk, no_recurse=recur_rel_fields)
                new_fields.append(Field(Field.REPEATED, field.name,
                                        pb_type, field))
                # TODO: Change msg name to avoid name collision at the ProtocolBuffer level
            else:
                # No data associated with the relation
                # Just like a foreign key but repeated
                if recurse_fk and not field.name in no_recurse:
                    m2m_dj_model = field.rel.to
                    pb_type = genMsg(m2m_dj_model.__name__, m2m_dj_model, recurse_fk=recurse_fk)
                    new_fields.append(Field(Field.REPEATED, field.name,
                                            pb_type, field))
                #else:
                    #pb_field_name += '_id'
                pb_type = DJ2PB.get(type(field))()
                new_fields.append(Field(Field.REPEATED, field.name + '_id',
                                        pb_type, field))
            #print field.rel.to, field.rel.through             
        else: # Not a Foreign Key
            # Add an enumeration for each django field with choices set
            if(field.choices):
                field_enum = Enum(field_name, [a for a,b in field.choices],
                                  "Generated from 'choices' for django model field: %s.%s" % (django_model_class.__name__, field.name))
                root_msg.enums.append(field_enum)
                pb_type = field_enum
            else:
                if isinstance(DJ2PB.get(type(field)), ProtocolBuffer.Message):
                    #TODO: The pb type here should be the generated pb2 message class or and instance of a metaclass
                    pb_type = DJ2PB.get(type(field))
                else:
                    pb_type = DJ2PB.get(type(field))()
            new_fields.append(Field(Field.OPTIONAL, field.name,
                                    pb_type, field))
        #pb_field = Field(pb_usage, pb_field_name, pb_type, field)
        #root_msg.addField(pb_field)
    root_msg.addField(*new_fields)
    return root_msg




#!/usr/bin/python
# Copyright (C) 2009  Las Cumbres Observatory <lcogt.net>
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''conversion.py - Handlers for mapping Django models and fields to
protocol buffer messages and fields

Authors: Zach Walker (zwalker@lcogt.net)
Dec 2009
'''


import decimal
from datetime import datetime, time
import traceback

from model import ProtocolBuffer
from django.db import models
from type_map import Date, Time
import type_map_base as types
import django_types_pb2 as proto



def _date_dj2pb(val, kwargs):
    '''Convert Django DateField to a protocol buffer Date message'''
    date = proto.Date()
    date.year = val.year
    date.month = val.month
    date.day = val.day
    return date

def _date_pb2dj(val, kwargs):
    '''Convert a protocol buffer Date msg to a Django DateField'''
    return datetime(val.year, val.month, val.day)
    
def _datetime_dj2pb(val, kwargs):
    '''Convert Django DateTimeField to a protocol buffer Date message'''
    date = proto.Date()
    date.year = val.year
    date.month = val.month
    date.day = val.day
    date.time.hour = val.hour
    date.time.minute = val.minute
    date.time.second = val.second
    #date.time.micro = val.microsecond
    return date

def _datetime_pb2dj(val, kwargs):
    '''Convert protocol buffer Date msg to Django DateTimeField'''
    return datetime(val.year, val.month, val.day, val.time.hour,
                    val.time.minute, val.time.second)#, val.time.micro)
    
def _time_dj2pb(val, kwargs):
    '''Convert  Django TimeField to a protocol buffer Time msg'''
    time = proto.Time()
    time.hour = val.hour
    time.minute = val.minute
    time.second = val.second
    #time.micro = val.microsecond
    return time

def _time_pb2dj(val, kwargs):
    '''Convert protocol buffer Time msg to Django TimeField'''
    file = models.FileField
    return time(val.hour, val.minute, val.second)#, val.micro)

def _m2m_pb2dj(val, kwargs):
    output_type = kwargs['output_type']
    model_type = output_type.to
    if not output_type.through is None:
        model_type = output_type.through
    if isinstance(model_type, str):
        model_type = models.__dict__[model_type]
    return model_type.objects.get(pk=val)


#def _file_dj2pb(val, kwargs):
#    '''Convert Django FileField to protocol buffer File msg'''
#    file = proto.File()
#    file.name = val.name
#    file.path = val.path
#    file.url = val.url
#    file.size = val.size 
#
#def _file_pb2dj(val, kwargs):
#    '''Convert protocol buffer File msg to a Django FileField'''
#    file = {'name':val.name, 'content':val.data, 'save':True}
#    return file

class Converter(object):
    
    def __init__(self):
        #TODO: Change django type portion of key to type rather than string
        conv_helpers = {}
        #Django seems to require float values input as a string
        conv_helpers[(types.PB_TYPE_DOUBLE,
                       models.DecimalField)] = lambda val, kwargs : str(val)
        #Convert Decimal back to a python float which is the same as a double
        #in protocol buffers apparently
        conv_helpers[(models.DecimalField,
                     types.PB_TYPE_DOUBLE)] = lambda val, kwargs : decimal.Decimal(val).__float__()
        #Help convert a Django DateTimeField into a consistent string format
        #for transport
        conv_helpers[(models.DateTimeField,
                      types.PB_TYPE_STRING)] = lambda val, kwargs : val.strftime("%Y-%m-%d %H:%M:%S")
        #Help convert a Django DateField into a consistent string format for transport
        conv_helpers[(models.DateField,
                      types.PB_TYPE_STRING)] = lambda val, kwargs : val.strftime("%Y-%m-%d")
        #Help convert a string into a Django DateTimeField
        conv_helpers[(types.PB_TYPE_STRING,
                      models.DateTimeField)] = \
                          lambda val, kwargs : datetime.strptime(val, "%Y-%m-%d %H:%M:%S")
        #Help convert a string into a  Django DateField
        conv_helpers[(types.PB_TYPE_STRING,
                      models.DateField)] = \
                          lambda val, kwargs : datetime.strptime(val, "%Y-%m-%d")
        conv_helpers[(models.FileField,
                      types.PB_TYPE_STRING)] = \
                          lambda val, kwargs : val.name
                          
        conv_helpers[(models.ImageField,
                      types.PB_TYPE_STRING)] = \
                          lambda val, kwargs : val.name
                          
        conv_helpers[(ProtocolBuffer.Enumeration, models.CharField)] = \
                          lambda val, kwargs: dict([(b,a) for a,b in kwargs['input_type'].dj2val.items()])[val]
        
        conv_helpers[(models.CharField, ProtocolBuffer.Enumeration)] = \
                          lambda val, kwargs: kwargs['output_type'].dj2val[val]
                          
        conv_helpers[(models.ForeignKey, types.PB_TYPE_INT32)] = \
                          lambda val, kwargs: val.pk
                          
        conv_helpers[(types.PB_TYPE_INT32, models.ForeignKey)] = \
                          lambda val, kwargs: kwargs['output_type'].related.parent_model.objects.get(pk=val).pk
                          
        conv_helpers[(models.ForeignKey, ProtocolBuffer.Message)] = \
                          lambda val, kwargs: self.toProtoMsg(kwargs['pb'], val, kwargs['module'])
                          #self.convert(kwargs['input_type'], kwargs['output_type'], val)
        conv_helpers[(ProtocolBuffer.Message, models.ForeignKey)] = \
                          lambda val, kwargs: self.toDjangoObj(kwargs['pb'], val)
                          #lambda val, kwargs: self.convert(kwargs['input_type'], kwargs['output_type'], val)
        conv_helpers[(models.ManyToManyField, types.PB_TYPE_INT32)] = \
                          lambda val, kwargs : val.pk
        conv_helpers[(types.PB_TYPE_INT32, models.ManyToManyField)] = \
                          lambda val, kwargs : _m2m_pb2dj(val, kwargs)
        conv_helpers[(models.ManyToManyField, ProtocolBuffer.Message)] = \
                          lambda val, kwargs : self.toProtoMsg(kwargs['pb'], val, kwargs['module'])
        conv_helpers[(ProtocolBuffer.Message, models.ManyToManyField)] = \
                          lambda val, kwargs : val
        
        #Help convert string to Django comma separated integer
        #conv_helpers[(types.PB_TYPE_STRING,
        #              models.CommaSeparatedIntegerField)] = \
        #                  lambda val, kwargs : eval(val)
        #conv_helpers[(models.CommaSeparatedIntegerField,
        #              types.PB_TYPE_STRING)] = \
        #                  lambda val, kwargs : str(val)
        
        conv_helpers[(models.DateTimeField, ProtocolBuffer.Message)] = _datetime_dj2pb
        conv_helpers[(models.DateField, ProtocolBuffer.Message)] = _date_dj2pb
        conv_helpers[(models.TimeField, ProtocolBuffer.Message)] = _time_dj2pb
        #helpers[(models.FileField, _buildFile())] = _file_dj2pb
        conv_helpers[(ProtocolBuffer.Message, models.DateField)] = _date_pb2dj
        conv_helpers[(ProtocolBuffer.Message, models.DateTimeField)] = _datetime_pb2dj
        conv_helpers[(ProtocolBuffer.Message, models.TimeField)] = _time_pb2dj
        #helpers[(_buildFile(), models.FileField)] = _file_pb2dj
        self.helpers = conv_helpers

    def addConvHelper(self, input_type, output_type, helper):
        '''Add a conversion helper to go from the supplied input type to
           output type by calling the helper function.  Helper should
           take have the format helper(val, kwargs).  Args passed to the
           helper by keyword will be input_type and output_type
        '''
        self.helpers[(input_type, output_type)] = helper

    def convert(self, input_type, output_type, val, **kwargs):
        '''Convert the input object to the output type and return an object of
           output type.  If there is not an existing mapping and conversion 
           helper between the type(input_obj) and output_type the input_obj
           will be returned unchanged
        '''
        #print input_type, output_type, val
        helper = self.helpers.get((type(input_type), type(output_type)), lambda x, kwargs : x)
        kwargs['input_type'] = input_type
        kwargs['output_type'] = output_type
        return helper(val, kwargs)

    def toDjangoObj(self, pb, obj, dest_obj=None):
            """ Take a protocol buffer object whose class was generated by
                this objcet and marshall a Django object
                Accepted Args:
                pb -- ProtocolBuffer object defining the relation 
                obj -- (google.protobuf.Message) The object to be converted
                dest_obj -- A django object of the proper type to which all
                            mapped fields will be copied
            """
            # Find the message mapping
            pb_msg_name = obj.__class__.__name__
            message = pb.messages[pb_msg_name]
            
            # Break message fields down into name keyed dict objects
            # by type of field 
            mapped_fields = dict([(field.name, field) for field in
                                  message.mapped_fields])
            composite_msg_fields = dict([(field.name, field) for field in
                                          message.message_fields])
            unmapped_fields = dict([(field.name, field) for field in
                                     message.fields])
    
            # Create a new django target message for this given type
            if message.django_model_class == None:
                raise Exception("No django model associated with pb msg type %s" %
                                pb_msg_name)
            if(not dest_obj is None):
                if isinstance(dest_obj, message.django_model_class):
                    # TODO: Add unittest for this case
                    dj_obj = dest_obj
                else:
                    # TODO: Add unittest for this case 
                    raise Exception("dest_obj type %s doesn't match model type %s" % 
                                    (type(dest_obj), message.django_model_class))
            else:
                dj_obj = message.django_model_class()
            
            # Iterate through msg fields and convert to django types
            for (field, val) in obj.ListFields():
                if field.name in composite_msg_fields:
                    # This is a composite field that requires special handling
                    pass
                elif field.name in unmapped_fields:
                    # ignore this field it isn't mapped
                    pass
                else:
                    if mapped_fields[field.name].usage == ProtocolBuffer.Field.REPEATED:
                        # repeated fields imply some kind of foreign key relationship
                        # requires special handling
                        # Django doesn't make it easy to add relation fields to an
                        # object without first saving it.
                        pass
                    else:
                        # This is an optional or required directly mappable field
                        mapped_field = mapped_fields[field.name]
                        dj_val = self.convert(mapped_field.pb_type,
                                              mapped_field.dj_type,
                                              val, pb=pb)
                        # For a foreign key, we set 'field_id' to the pk of the related value 
                        setattr(dj_obj, mapped_field.name, dj_val)
            return dj_obj
    
    
    def toProtoMsg(self, pb, obj, module, dest_obj=None):
        """ Take a django object for which a protocol buffer message
            has been generated and return a related protocol buffer
            message.  Since the python implementation of protocol
            buffers uses factory methods to create field instances
            of nested messages, use the dest_obj arg to
            do the type conversion for you.
            Ex. pbuf.toProtoMsg(obj=django_obj, dest_obj=my_msg.field.add())
            Accepted Args:
            pb -- ProtocolBuffer object defining the relation 
            obj -- (django Model) The object to be converted
            module -- protocol buffer module generate from .proto file modeled by pb
            dest_obj -- (Message) An instance of the message class
                        that is related to the django object arg.
                        Instead of creating a new object this obj
                        will be loaded with data and returned
        """
        # Import the message type and instantiate if necessary
        dj_type = type(obj)
        # Try to get the type using the django class name
        message = pb.messages.get(dj_type)
        msg_type = getattr(module, message.name, None)
        if msg_type == None:
            raise Exception("No mapping available for django type %s." % dj_type)
        
            
        if dest_obj is None:
            protoMsg = msg_type()
        else:
            protoMsg = dest_obj
        
        # Get the pb message type and convert the django message    
        for field in message.mapped_fields:
            if field.usage == ProtocolBuffer.Field.REPEATED:
                # Must be a ManyToMany type relation
                val_list = []
                if not field.dj_type.rel.through is None:
                    # Get the association model
                    assoc_model = field.dj_type.rel.through_model
                    # Access it through the parent type member names 'modelclass_set'
                    val_list = getattr(obj, assoc_model.__name__.lower() + '_set').all()
                else:
                    # No through model use the related objects directly
                    val_list = getattr(obj, field.dj_type.name).all()
                rep_field = getattr(protoMsg, field.name)
                
                # Iterate thorugh and convert each value independantly
                for val in val_list:
                    try:
                        pb_val = self.convert(field.dj_type,
                                          field.pb_type,
                                          val, pb=pb, module=module)
                    except Exception, e:
                        #print field.name, field.dj_type, field.pb_type
                        #traceback.print_exc()
                        raise e
                    if isinstance(field.pb_type, ProtocolBuffer.Message):
                        rep_field.add().CopyFrom(pb_val)
                    else:
                        rep_field.append(pb_val)
            else:
                try:
                    val = getattr(obj, field.dj_type.name)
                except Exception, e:
                    #print message.name, type(obj), field.name, type(field.dj_type.name), field.dj_type.name, field.dj_type, field.pb_type
                    #traceback.print_exc()
                    raise e
                if  val != None and val != "":
                    # TODO: Check if field is repeated and if so get all objects
                    # that need to be converted
                    pb_val = self.convert(field.dj_type,
                                          field.pb_type,
                                          val, pb=pb, module=module)
                    # Use CopyFrom if it field is a composite type
                    if isinstance(field.pb_type, ProtocolBuffer.Message):
                        getattr(protoMsg, field.name).CopyFrom(pb_val)
                    else:
                        # print field.name
                        try:
                            setattr(protoMsg, field.name, pb_val)
                        except Exception, e:
                            #print field.name, field.dj_type, field.pb_type, val, val.pk, val.fk_test,  pb_val
                            #traceback.print_exc()
                            raise e
        return protoMsg


#!/usr/bin/python
# Copyright (C) 2009  Las Cumbres Observatory <lcogt.net>
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''pbandj.py - Functions for generating .proto files and
pyhton modules.

Authors: Zach Walker (zwalker@lcogt.net)
Dec 2009
'''


from os import system

from type_map import DJANGO_PROTO
from model import ProtocolBuffer

def _genProto(proto, path="."):
    f = open(path + "/" + proto.proto_filename(), 'w')
    f.write(proto.__str__())
    f.close()

def genProto(proto, path="."):
    if proto != DJANGO_PROTO and not(DJANGO_PROTO in proto.imports):
        proto.imports.append(DJANGO_PROTO)
    protos = (DJANGO_PROTO, proto)
    for proto in protos:
        _genProto(proto, path)


def genMod(proto, path="."):
    """ Generate a python module for this Protocol Buffer.
        The default name is genereated_pb2.py unless specified.
        Accepted Arguments:
        path -- (String) The path, relative or fully qualified,
                to the directory where the generated module will
                be created
    """
    genProto(proto, path)
    modules = proto.imports + [proto]
    for mod in modules:
        if type(mod) is ProtocolBuffer:
            mod = mod.proto_filename()
        system("protoc --python_out=" + path +
               " --proto_path=" + path + " " +
               path + "/" + mod)
               
    return proto.module_name + "_pb2"


#!/usr/bin/python
# Copyright (C) 2009  Las Cumbres Observatory <lcogt.net>
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''test_pbandj.py - Unittests for pbandj

Authors: Zach Walker (zwalker@lcogt.net)
Dec 2009
'''
import unittest
import os
from datetime import datetime

# Set the enviroment variable Django looks at to find db settings
from django.conf import ENVIRONMENT_VARIABLE
os.environ[ENVIRONMENT_VARIABLE] = 'django_test.settings'

from django.db import models as django_models
from pbandj.model import ProtocolBuffer
from pbandj.type_map import DJ2PB, genMsg
from pbandj.conversion import Converter
from pbandj import pbandj
from django_test.test_app import models
from protobuf import RpcService


class MessageRenameTestCase(unittest.TestCase):
    '''Test mapping when message name does not match
    django type name
    '''
    global proto
     
    def setUp(self):
        self.module_name = "simple_test"
        self.pb = ProtocolBuffer(self.module_name)
        self.simple = genMsg('SimpleRename', models.Simple)
        self.pb.addMessage(self.simple)
        mod_name = pbandj.genMod(self.pb)
        global proto
        proto = __import__(mod_name)
        self.con = Converter()
        
    def test_mappingExists(self):
        self.assert_(not self.pb.messages.get('SimpleRename') is None,
                     'No mapping by message name to message object')
        self.assert_(not self.pb.messages.get(models.Simple) is None,
                     'No mapping by django type to message object')
        self.assertEqual(self.pb.messages['SimpleRename'],
                         self.pb.messages[models.Simple],
                         'mapping from string not equal to mapping from django type')
    
    def test_invertible(self):
        msg = proto.SimpleRename()
        msg.val = 22
        django = self.con.toDjangoObj(self.pb, msg)
        msg = self.con.toProtoMsg(self.pb, django, proto)
         

class OneOfEverythingTestCase(unittest.TestCase):
    
    def setUp(self):
        self.module_name = "ooe_test"
        self.pb = ProtocolBuffer(self.module_name)
        self.ooe = genMsg('OneOfEverything',
                                        models.OneOfEverything)
        self.pb.addMessage(self.ooe)
        
        self.django_class_fields = [field for field in 
                               models.OneOfEverything._meta.local_fields]
        self.pb_field_names = [f.name for f in self.ooe.mapped_fields]

    def test_genMsg_field_types(self):
        ''' Ensure that there is a type map definition for each Django field
            type
        '''
        for field in self.django_class_fields:
            self.assert_(DJ2PB.get(type(field)) != None,
                         "No protocol buffer type mapped to %s Django type" % 
                         type(field).__name__)
    
    def test_genMsg_fields_exist(self):
        ''' Ensure that all fields from the django type exist in the generated
            protocol buffer type and their type matches the type map
        '''
        pb_fields_by_name = dict([(f.name, f) for f in self.ooe.mapped_fields])
        for field in self.django_class_fields:
            self.assert_(self.pb_field_names.count(field.name) > 0,
                         "Django field %s does not exist in protocol buffer message" % field.name)
            pb_field = pb_fields_by_name[field.name]
            dj_type = type(field)
            if isinstance(pb_field.pb_type, ProtocolBuffer.Message):
                self.assert_(pb_field.pb_type == DJ2PB[dj_type],
                             "Protocol Buffer field type %s does not match type %s specified in type map for Django type %s" %
                             (pb_field.pb_type, DJ2PB[dj_type],
                              dj_type.__name__))
            else:
                self.assert_(type(pb_field.pb_type) == DJ2PB[dj_type],
                             "Protocol Buffer field type %s does not match type %s specified in type map for Django type %s" %
                             (type(pb_field.pb_type), DJ2PB[dj_type],
                              dj_type.__name__))
    
    def test_genMod(self):
        '''Test that the generated mod name is correct and the module is 
           importable
        '''
        mod_name = pbandj.genMod(self.pb)
        self.assertEqual(mod_name, self.module_name + "_pb2",
                         "Module name does not match name passed in to ProtocolBuffer() constructor")
        try:
            proto = __import__(mod_name)
        except ImportError, ie:
            self.assert_(False,
                     "Failed to import generated python module")


class OneOfEverythingUsageTestCase(unittest.TestCase):
    
    proto = None
    
    def setUp(self):
        self.module_name = "ooe_test"
        self.pb = ProtocolBuffer(self.module_name)
        self.ooe = genMsg('OneOfEverything',
                                        models.OneOfEverything)
        self.pb.addMessage(self.ooe)
        
        self.django_class_fields = [field for field in 
                               models.OneOfEverything._meta.local_fields]
        self.pb_field_names = [f.name for f in self.ooe.mapped_fields]
        mod_name = pbandj.genMod(self.pb)
        global proto
        proto = __import__(mod_name)
        
        ooe = proto.OneOfEverything()
        ooe.bool_test = True
        ooe.char_test = "ABC123"
        ooe.comma_test = "A,B,C"
        ooe.date_test.year = 1980
        ooe.date_test.month = 6
        ooe.date_test.day = 10
        ooe.date_time_test.year = 1989
        ooe.date_time_test.month = 9
        ooe.date_time_test.day = 19
        ooe.date_time_test.time.hour = 1
        ooe.date_time_test.time.minute = 2
        ooe.date_time_test.time.second = 3
        ooe.decimal_test = 12.34
        ooe.float_test = 56.789
        ooe.email_test = 'zman@namz.com'
        ooe.file_test = "path/to/file"
        ooe.image_test = "/path/to/image"
        ooe.int_test = -1
        ooe.ip_test = '123.456.789.123'
        ooe.null_bool_test = False
        ooe.pos_int_test = 1
        ooe.pos_sm_int_test = 1
        ooe.slug_test = 'This is a slug'
        ooe.sm_int_test = -1
        ooe.text_test = 'This is some text'
        ooe.time_test.hour = 12
        ooe.time_test.minute = 13
        ooe.time_test.second = 14
        ooe.url_test = 'http://www.lcogt.net'
        ooe.xml_test = '<abc>123</abc>'
        self.ooe = ooe
        self.converter = Converter()
        
        
    def test_conversion_invertible(self):
        '''Make sure conversion is invertible'''
        dj_obj = self.converter.toDjangoObj(self.pb, self.ooe)
        pb_obj = self.converter.toProtoMsg(self.pb, dj_obj, proto)
        self.assertEqual(self.ooe, pb_obj)
        
    def test_save_invertible(self):
        '''Make sure conversion is invertible going through database'''
        dj_obj = self.converter.toDjangoObj(self.pb, self.ooe)
        dj_obj.save()
        pb_from_db = self.converter.toProtoMsg(self.pb,
                                          type(dj_obj).objects.get(pk=dj_obj.id),
                                          proto)
        self.ooe.id = dj_obj.id
        self.assertEqual(self.ooe, pb_from_db)
        
class EnumTestCase(unittest.TestCase):
    proto = None
    def setUp(self):
        self.module_name = "enum_test"
        self.pb = ProtocolBuffer(self.module_name)
        self.emsg= genMsg('EnumTest',
                                        models.EnumTest)
        self.pb.addMessage(self.emsg)
        self.django_class_fields = [field for field in 
                               models.EnumTest._meta.local_fields]
        self.pb_field_names = [f.name for f in self.emsg.mapped_fields]
        mod_name = pbandj.genMod(self.pb)
        global proto
        proto = __import__(mod_name)
        emsg = proto.EnumTest()
        emsg.enum_test = proto.EnumTest.ENUM_TEST_VAL1
        self.emsg = emsg
        self.converter = Converter()
        
        enum_db = models.EnumTest(enum_test='Val1')
        enum_db.save()
        self.enum_db = enum_db
        
    def test_toDjangoObj(self):
        dj_obj = self.converter.toDjangoObj(self.pb, self.emsg)
        self.assertEqual(dj_obj.enum_test, self.pb.messages['EnumTest'].enums[0].dj2val[proto.EnumTest.ENUM_TEST_VAL1])
    
    def test_conversion_invertible(self):
        '''Make sure conversion is invertible'''
        dj_obj = self.converter.toDjangoObj(self.pb, self.emsg)
        pb_obj = self.converter.toProtoMsg(self.pb, dj_obj, proto)
        self.assertEqual(self.emsg, pb_obj)
    
    def test_save_invertible(self):
        '''Make sure conversion is invertible going through database'''
        dj_obj = self.converter.toDjangoObj(self.pb, self.emsg)
        dj_obj.save()
        pb_from_db = self.converter.toProtoMsg(self.pb,
                                          type(dj_obj).objects.get(pk=dj_obj.id),
                                          proto)
        self.emsg.id = dj_obj.id
        self.assertEqual(self.emsg, pb_from_db)
        
class ServiceTestCase(unittest.TestCase):
    
    proto = None
    services_started = False
    
    def setUp(self):
        self.pb = ProtocolBuffer("service_test")
        simple_msg = genMsg("Simple", models.Simple)
        self.pb.addMessage(simple_msg)
        self.service_port = 4567
        
        def SimpleHandler(service_impl, controller, request, done):
            request.val += 1
            done.run(request)
        
        def SimpleHandler2(service_impl, controller, request, done):
            '''Docstring from Handler2'''
            request.val += 5
            done.run(request)
            
        self.pb.addRpc('SimpleService', 'TestRpc', simple_msg,
                                   simple_msg, SimpleHandler,
                                   'A line of documentation')
        self.pb.addRpc('SimpleService', 'TestRpc2', simple_msg,
                                   simple_msg, SimpleHandler2)
        self.pb.addRpc('SimpleService2', 'TestRpc', simple_msg,
                                   simple_msg, SimpleHandler)
        mod_name = pbandj.genMod(self.pb)
        global proto
        proto = __import__(mod_name)
        if not ServiceTestCase.services_started:
            self.pb.services['SimpleService'].start(proto, self.service_port, True)
            self.pb.services['SimpleService2'].start(proto, self.service_port + 1, True)
            ServiceTestCase.services_started = True

        
    def test_RpcService(self):
        client = RpcService(proto.SimpleService_Stub, self.service_port,
                            "localhost")
        test_msg = proto.Simple()
        test_msg.val = 1
        response = client.TestRpc(test_msg, timeout=5000)
        self.assertEqual(response.val, test_msg.val + 1,
                         "Rpc test failed")
    
    def test_n_rpc_same_service(self):
        client = RpcService(proto.SimpleService_Stub, self.service_port,
                            "localhost")
        test_msg = proto.Simple()
        test_msg.val = 1
        response = client.TestRpc(test_msg, timeout=5000)
        self.assertEqual(response.val, test_msg.val + 1,
                         "Rpc test failed")
        response = client.TestRpc2(test_msg, timeout=5000)
        self.assertEqual(response.val, test_msg.val + 5,
                         "Rpc test failed")
        
    def test_n_services(self):
        client1 = RpcService(proto.SimpleService_Stub, self.service_port,
                            "localhost")
        client2 = RpcService(proto.SimpleService2_Stub, self.service_port + 1,
                            "localhost")
        test_msg = proto.Simple()
        test_msg.val = 1
        response = client1.TestRpc(test_msg, timeout=5000)
        self.assertEqual(response.val, test_msg.val + 1,
                         "Rpc test failed")
        response = client2.TestRpc(test_msg, timeout=5000)
        self.assertEqual(response.val, test_msg.val + 1,
                         "Rpc test failed")   
        
        
class ForeignKeyTestCase(unittest.TestCase):
    proto = None
    
    def setUp(self):
        self.module_name = "fk_test"
        self.pb = ProtocolBuffer(self.module_name)
        self.fkt = genMsg('ForeignKeyTest',
                                        models.ForeignKeyTest)
        self.pb.addMessage(self.fkt)
        
        self.django_class_fields = [field for field in 
                               models.ForeignKeyTest._meta.local_fields]
        self.pb_fields_by_name = dict([(f.name, f) for f in self.fkt.mapped_fields])
        mod_name = pbandj.genMod(self.pb)
        global proto
        proto = __import__(mod_name)
        self.simple = models.Simple()
        self.simple.val = 1234
        self.simple.save()
        self.con = Converter()
    
    def test_genMsg(self):
        ''' Ensure that there is a type map definition for each Django field
            type
        '''
        for field in self.django_class_fields:
            self.assert_(DJ2PB.get(type(field)) != None,
                         "No protocol buffer type mapped to %s Django type" % 
                         type(field).__name__)
        
    def test_genMsg_fields_exist(self):
        ''' Ensure that all fields from the django type exist in the generated
            protocol buffer type and their type matches the type map
        '''
        for field in self.django_class_fields:
            self.assert_(self.pb_fields_by_name.has_key(field.name),
                         "Django field %s does not exist in protocol buffer message" % field.name)
            pb_field = self.pb_fields_by_name[field.name]
            dj_type = type(field)
            expected_type = [DJ2PB[dj_type]]
            if (dj_type == django_models.ForeignKey or dj_type == django_models.ManyToManyField):
                expected_type.append(ProtocolBuffer.Message)
            self.assert_(type(pb_field.pb_type) in expected_type,
                         "Protocol Buffer field type %s does not match type %s specified in type map for Django type %s" %
                         (type(pb_field.pb_type), DJ2PB[dj_type],
                          dj_type.__name__))
            
    def test_invertible(self):
        test = proto.ForeignKeyTest()
        test.fkey_test_id = self.simple.pk
        dj_test = self.con.toDjangoObj(self.pb, test)
        pb_test = self.con.toProtoMsg(self.pb, dj_test, proto)
        self.assertEqual(test.fkey_test_id, pb_test.fkey_test_id,
                         "Foreign key test objects not equal after conversion")
        
class ForeignKeyRecursionTestCase(unittest.TestCase):
    proto = None
    
    def setUp(self):
        self.module_name = "fk_recusion_test"
        self.pb = ProtocolBuffer(self.module_name)
        self.fkt = genMsg('ForeignKeyTest',
                           models.ForeignKeyTest,
                           recurse_fk = True)
        self.pb.addMessage(self.fkt)
        
        self.django_class_fields = [field for field in 
                               models.ForeignKeyTest._meta.local_fields]
        self.pb_fields_by_name = dict([(f.name, f) for f in self.fkt.mapped_fields])
        mod_name = pbandj.genMod(self.pb)
        global proto
        proto = __import__(mod_name)
        self.simple = models.Simple()
        self.simple.val = 1234
        self.simple.save()
        self.con = Converter()
    
    def test_genMsg(self):
        ''' Ensure that there is a type map definition for each Django field
            type
        '''
        for field in self.django_class_fields:
            self.assert_(DJ2PB.get(type(field)) != None,
                         "No protocol buffer type mapped to %s Django type" % 
                         type(field).__name__)
        
    def test_genMsg_fields_exist(self):
        ''' Ensure that all fields from the django type exist in the generated
            protocol buffer type and their type matches the type map
        '''
        for field in self.django_class_fields:
            self.assert_(self.pb_fields_by_name.has_key(field.name),
                         "Django field %s does not exist in protocol buffer message" % field.name)
            pb_field = self.pb_fields_by_name[field.name]
            dj_type = type(field)
            expected_type = [DJ2PB[dj_type]]
            if (dj_type == django_models.ForeignKey or dj_type == django_models.ManyToManyField):
                expected_type.append(ProtocolBuffer.Message)
            self.assert_(type(pb_field.pb_type) in expected_type,
                         "Protocol Buffer field type %s does not match type %s specified in type map for Django type %s" %
                         (type(pb_field.pb_type), DJ2PB[dj_type],
                          dj_type.__name__))
            
    def test_invertible(self):
        test = proto.ForeignKeyTest()
        test.fkey_test_id = self.simple.pk
        dj_test = self.con.toDjangoObj(self.pb, test)
        pb_test = self.con.toProtoMsg(self.pb, dj_test, proto)
        self.assertEqual(test.fkey_test_id, pb_test.fkey_test_id,
                         "Foreign key test objects not equal after conversion")
        
    def test_save_invertible(self):
        test = proto.ForeignKeyTest()
        self.con.toProtoMsg(self.pb, self.simple, proto, test.fkey_test)
        dj_test = self.con.toDjangoObj(self.pb, test)
        dj_test.save()
        pb_test = self.con.toProtoMsg(self.pb, models.ForeignKeyTest.objects.get(pk=dj_test.pk), proto)
        pb_test.ClearField('id')
        self.assertEqual(test.fkey_test, pb_test.fkey_test,
                         "Foreign key test objects not equal after save")
        
class ManyToManyTestCase(unittest.TestCase):
    
    def setUp(self):
        self.module_name = "m2m_test"
        self.pb = ProtocolBuffer(self.module_name)
        self.m2m = genMsg('ManyToManyTest',
                                        models.ManyToManyTest)
        self.pb.addMessage(self.m2m)
        
        self.django_class_fields = [field for field in 
                               models.ManyToManyTest._meta.local_fields +
                               models.ManyToManyTest._meta.many_to_many]
        self.pb_fields_by_name = dict([(f.name, f) for f in self.m2m.mapped_fields])
        mod_name = pbandj.genMod(self.pb)
        global proto
        proto = __import__(mod_name)
        self.simple1 = models.Simple()
        self.simple1.val = 1234
        self.simple1.save()
        self.simple2 = models.Simple()
        self.simple2.val = 5678
        self.simple2.save()
        self.con = Converter()
        
    def test_genMsg(self):
        ''' Ensure that there is a type map definition for each Django field
            type
        '''
        for field in self.django_class_fields:
            self.assert_(DJ2PB.get(type(field)) != None,
                         "No protocol buffer type mapped to %s Django type" % 
                         type(field).__name__)
    
    def test_genMsg_fields_exist(self):
        ''' Ensure that all fields from the django type exist in the generated
            protocol buffer type and their type matches the type map
        '''
        for field in self.django_class_fields:
            self.assert_(self.pb_fields_by_name.has_key(field.name),
                         "Django field %s does not exist in protocol buffer message" % field.name)
            pb_field = self.pb_fields_by_name[field.name]
            dj_type = type(field)
            expected_type = [DJ2PB[dj_type]]
            if (dj_type == django_models.ForeignKey or dj_type == django_models.ManyToManyField):
                expected_type.append(ProtocolBuffer.Message)
            self.assert_(type(pb_field.pb_type) in expected_type,
                         "Protocol Buffer field type %s does not match type %s specified in type map for Django type %s" %
                         (type(pb_field.pb_type), DJ2PB[dj_type],
                          dj_type.__name__))
    
    def test_toDjangoObj(self):
        pb_test = proto.ManyToManyTest()
        pb_test.test_val = 1
        pb_test.m2m_test_id.append(2)
        dj_test = self.con.toDjangoObj(self.pb, pb_test)
        self.assertEqual(pb_test.test_val, dj_test.test_val,
                         'Django object does not match source Protocol Buffer Message')
    
    def test_toProtoMsg(self):
        dj_test = models.ManyToManyTest()
        dj_test.test_val = 1
        dj_test.save()
        dj_test.m2m_test.add(self.simple1) 
        dj_test.m2m_test.add(self.simple2)       
#        dj_test2 = models.ManyToManyTest()
#        dj_test2.test_val = 2
#        dj_test2.save()
#        dj_test2.m2m_test.add(self.simple) 
        pb_test = self.con.toProtoMsg(self.pb, dj_test, proto)
        self.assert_(len(pb_test.m2m_test_id) == 2,
                     "Added 2 objects to the relation but found %d" % len(pb_test.m2m_test_id))
        self.assertEqual(pb_test.m2m_test_id[0], self.simple1.pk,
                         "1st assoc object doesn't match the original")
        self.assertEqual(pb_test.m2m_test_id[1], self.simple2.pk,
                         "2nd assoc object doesn't match the original")
        
class ManyToManyThrough_Recurse_TestCase(unittest.TestCase):
    
    def setUp(self):
        self.module_name = "m2m_through_test_recurse"
        self.pb = ProtocolBuffer(self.module_name)
        self.m2m = genMsg('ManyToManyThroughTest',
                                        models.ManyToManyThroughTest, recurse_fk=True)
        self.pb.addMessage(self.m2m)
        
        self.django_class_fields = [field for field in 
                               models.ManyToManyThroughTest._meta.local_fields +
                               models.ManyToManyThroughTest._meta.many_to_many]
        
        self.django_assoc_class_fields = [field for field in 
                               models.M2MAssocTest._meta.local_fields +
                               models.M2MAssocTest._meta.many_to_many]
        
        self.pb_fields_by_name = dict([(f.name, f) for f in self.m2m.mapped_fields])
        self.pb_assoc_fields_by_name = dict([(f.name, f) for f in self.pb_fields_by_name['m2m_test'].pb_type.mapped_fields])
        mod_name = pbandj.genMod(self.pb)
        global proto
        proto = __import__(mod_name)
        self.simple = models.Simple()
        self.simple.val = 1234
        self.simple.save()
        self.con = Converter()
        
    def test_genMsg(self):
        ''' Ensure that there is a type map definition for each Django field
            type
        '''
        for field in self.django_class_fields:
            self.assert_(DJ2PB.get(type(field)) != None,
                         "No protocol buffer type mapped to %s Django type" % 
                         type(field).__name__)
    
    def test_genMsg_fields_exist(self):
        ''' Ensure that all fields from the django type exist in the generated
            protocol buffer type and their type matches the type map
        '''
        for field in self.django_class_fields:
            self.assert_(self.pb_fields_by_name.has_key(field.name),
                         "Django field %s does not exist in protocol buffer message" % field.name)
            pb_field = self.pb_fields_by_name[field.name]
            dj_type = type(field)
            expected_type = [DJ2PB[dj_type]]
            if (dj_type == django_models.ForeignKey or dj_type == django_models.ManyToManyField):
                expected_type.append(ProtocolBuffer.Message)
            self.assert_(type(pb_field.pb_type) in expected_type,
                         "Protocol Buffer field type %s does not match type %s specified in type map for Django type %s" %
                         (type(pb_field.pb_type), DJ2PB[dj_type],
                          dj_type.__name__))
    
    def test_genMsg_through_fields_exist(self):
        for field in self.django_assoc_class_fields:
            self.assert_(self.pb_assoc_fields_by_name.has_key(field.name),
                         "Django field %s does not exist in protocol buffer message for through relation" % field.name)
            pb_field = self.pb_assoc_fields_by_name[field.name]
            dj_type = type(field)
            expected_type = [DJ2PB[dj_type]]
            if dj_type == django_models.ForeignKey or dj_type == django_models.ManyToManyField:
                expected_type.append(ProtocolBuffer.Message)
            self.assert_(type(pb_field.pb_type) in expected_type,
                         "Protocol Buffer field type %s does not match type %s specified in type map for Django type %s" %
                         (type(pb_field.pb_type), DJ2PB[dj_type],
                          dj_type.__name__))
            if dj_type == django_models.ForeignKey and field.name == 'm2m_fk':
                self.assert_(pb_field.pb_type != self.m2m,
                         "Assoc class field creates a circular relationship")
            if dj_type == django_models.ForeignKey and field.name == 'simple_fk':
                self.assert_(isinstance(pb_field.pb_type, ProtocolBuffer.Message),
                         "recurse_fk was True but genMsg failed to recurse into fk type")
            
    def test_toProtoMsg(self):
        dj_test = models.ManyToManyThroughTest()
        dj_test.test_val = 1
        dj_test.save()
        assoc_test = models.M2MAssocTest()
        assoc_test.assoc_test = 1
        assoc_test.simple_fk = self.simple
        assoc_test.m2m_fk = dj_test
        assoc_test.save()
        assoc_test2 = models.M2MAssocTest()
        assoc_test2.assoc_test = 2
        assoc_test2.simple_fk = self.simple
        assoc_test2.m2m_fk = dj_test
        assoc_test2.save()
        pb_test = self.con.toProtoMsg(self.pb, dj_test, proto)
        self.assert_(len(pb_test.m2m_test) == 2,
                     "Added 2 objects to the relation but found %d" % len(pb_test.m2m_test))
        self.assertEqual(pb_test.m2m_test[0],
                         self.con.toProtoMsg(self.pb, assoc_test, proto),
                         "Conversion of 1st assoc object doesn't match stand alone conversion of the same object")
        self.assertEqual(pb_test.m2m_test[1],
                         self.con.toProtoMsg(self.pb, assoc_test2, proto),
                         "Conversion of 2nd assoc object doesn't match stand alone conversion of the same object")
    

class ManyToManyThrough_NoRecurse_TestCase(unittest.TestCase):
    
    def setUp(self):
        self.module_name = "m2m_through_test_no_recurse"
        self.pb = ProtocolBuffer(self.module_name)
        self.m2m = genMsg('ManyToManyThroughTest',
                                        models.ManyToManyThroughTest, recurse_fk=False)
        self.pb.addMessage(self.m2m)
        
        self.django_class_fields = [field for field in 
                               models.ManyToManyThroughTest._meta.local_fields +
                               models.ManyToManyThroughTest._meta.many_to_many]
        
        self.django_assoc_class_fields = [field for field in 
                               models.M2MAssocTest._meta.local_fields +
                               models.M2MAssocTest._meta.many_to_many]
        
        self.pb_fields_by_name = dict([(f.name, f) for f in self.m2m.mapped_fields])
        self.pb_assoc_fields_by_name = dict([(f.name, f) for f in self.pb_fields_by_name['m2m_test'].pb_type.mapped_fields])
        mod_name = pbandj.genMod(self.pb)
        global proto
        proto = __import__(mod_name)
        self.simple = models.Simple()
        self.simple.val = 1234
        self.simple.save()
        self.con = Converter()
        
    def test_genMsg(self):
        ''' Ensure that there is a type map definition for each Django field
            type
        '''
        for field in self.django_class_fields:
            self.assert_(DJ2PB.get(type(field)) != None,
                         "No protocol buffer type mapped to %s Django type" % 
                         type(field).__name__)
        for field in self.django_assoc_class_fields:
            self.assert_(DJ2PB.get(type(field)) != None,
                         "No protocol buffer type mapped to %s Django type" % 
                         type(field).__name__)
    
    def test_genMsg_fields_exist(self):
        ''' Ensure that all fields from the django type exist in the generated
            protocol buffer type and their type matches the type map
        '''
        for field in self.django_class_fields:
            self.assert_(self.pb_fields_by_name.has_key(field.name),
                         "Django field %s does not exist in protocol buffer message" % field.name)
            pb_field = self.pb_fields_by_name[field.name]
            dj_type = type(field)
            expected_type = [DJ2PB[dj_type]]
            if (dj_type == django_models.ForeignKey or dj_type == django_models.ManyToManyField):
                expected_type.append(ProtocolBuffer.Message)
            self.assert_(type(pb_field.pb_type) in expected_type,
                         "Protocol Buffer field type %s does not match type %s specified in type map for Django type %s" %
                         (type(pb_field.pb_type), DJ2PB[dj_type],
                          dj_type.__name__))

    
    def test_genMsg_through_fields_exist(self):
        for field in self.django_assoc_class_fields:
            self.assert_(self.pb_assoc_fields_by_name.has_key(field.name),
                         "Django field %s does not exist in protocol buffer message for through relation" % field.name)
            pb_field = self.pb_assoc_fields_by_name[field.name]
            dj_type = type(field)
            expected_type = [DJ2PB[dj_type]]
            if dj_type == django_models.ForeignKey or dj_type == django_models.ManyToManyField:
                expected_type.append(ProtocolBuffer.Message)
            self.assert_(type(pb_field.pb_type) in expected_type,
                         "Protocol Buffer field type %s does not match type %s specified in type map for Django type %s" %
                         (type(pb_field.pb_type), DJ2PB[dj_type],
                          dj_type.__name__))
            if dj_type == django_models.ForeignKey and field.name == 'm2m_fk':
                self.assert_(pb_field.pb_type != self.m2m,
                         "Assoc class field creates a circular relationship")
            if dj_type == django_models.ForeignKey and field.name == 'simple_fk':
                self.assert_(not isinstance(pb_field.pb_type, ProtocolBuffer.Message),
                         "Recursive relationship found when recurse_fk was False")
            
    def test_toProtoMsg(self):
        dj_test = models.ManyToManyThroughTest()
        dj_test.test_val = 1
        dj_test.save()
        assoc_test = models.M2MAssocTest()
        assoc_test.assoc_test = 1
        assoc_test.simple_fk = self.simple
        assoc_test.m2m_fk = dj_test
        assoc_test.save()
        assoc_test2 = models.M2MAssocTest()
        assoc_test2.assoc_test = 2
        assoc_test2.simple_fk = self.simple
        assoc_test2.m2m_fk = dj_test
        assoc_test2.save()
        pb_test = self.con.toProtoMsg(self.pb, dj_test, proto)
        self.assert_(len(pb_test.m2m_test) == 2,
                     "Added 2 objects to the relation but found %d" % len(pb_test.m2m_test))
        self.assertEqual(pb_test.m2m_test[0],
                         self.con.toProtoMsg(self.pb, assoc_test, proto),
                         "Conversion of 1st assoc object doesn't match stand alone conversion of the same object")
        self.assertEqual(pb_test.m2m_test[1],
                         self.con.toProtoMsg(self.pb, assoc_test2, proto),
                         "Conversion of 2nd assoc object doesn't match stand alone conversion of the same object")
            
    
             
class GenModTestCase(unittest.TestCase):
    
    def setup(self):
        pass
    
if __name__ == '__main__':
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(OneOfEverythingTestCase))
    suite.addTest(unittest.makeSuite(MessageRenameTestCase))
    suite.addTest(unittest.makeSuite(EnumTestCase))
    suite.addTest(unittest.makeSuite(OneOfEverythingUsageTestCase))
    suite.addTest(unittest.makeSuite(ServiceTestCase))
    suite.addTest(unittest.makeSuite(ForeignKeyTestCase))
    suite.addTest(unittest.makeSuite(ForeignKeyRecursionTestCase))
    suite.addTest(unittest.makeSuite(ManyToManyTestCase))
    suite.addTest(unittest.makeSuite(ManyToManyThrough_Recurse_TestCase))
    suite.addTest(unittest.makeSuite(ManyToManyThrough_NoRecurse_TestCase))
    unittest.TextTestRunner(verbosity=0).run(suite)
                
        
        
    



#!python

#    Copyright (C) 2009  Zachary Walker
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

from time import time, sleep
#from protobuf.channel import SocketRpcChannel
#from pbandj.rpc import ProtoBufRpcRequest
from protobuf import RpcService
import generated_pb2 as proto
    
class AddressBook(object):
    """ A wrapper class for a protocol buffer AddressBook message """
    
    def __init__(self):
        self.service = RpcService(proto.AddressService_Stub, 8091, 'localhost')

    def add_contact(self, name, email, phone, birthday, favorite, groups=[]):
        """ Add a contact to the address book """
        msg = proto.AddressBook()
        msg.name = name
        msg.email = email
        msg.phone = phone
        msg.birthday.year = 1976
        msg.birthday.month = 10
        msg.birthday.day = 17
        msg.favorite_flag = favorite
        for group in groups:
            msg.group.append(group)
        return self.service.add_contact(msg, timeout=5000)
    
    def get_group(self, group_name):
        """ Get the contacts in a group """
        msg = proto.ContactGroup()
        msg.group_name = group_name
        return self.service.get_group(msg, timeout=5000)
